/*============================================================================
  File:     02_AutomaticPlanCorrection.sql

  SQL Server Versions: 2017, 2019, Azure SQL Database
------------------------------------------------------------------------------
  Written by Erin Stellato, Microsoft
  
  (c) 2022, Erin Stellato. All rights reserved.

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [master];
GO
RESTORE DATABASE [WideWorldImportersB] 
	FROM  DISK = N'C:\Backups\WWI_Standard.bak' 
		WITH  FILE = 1,  
	MOVE N'WWI_Primary' 
		TO N'C:\Databases\WideWorldImportersLarge\WideWorldImporters.mdf',  
	MOVE N'WWI_UserData' 
		TO N'C:\Databases\WideWorldImportersLarge\WideWorldImporters_UserData.ndf',  
	MOVE N'WWI_Log' 
		TO N'C:\Databases\WideWorldImportersLarge\WideWorldImporters.ldf',  
	NOUNLOAD,  
	REPLACE,  
	STATS = 5;
GO


/*
	Generate CustomerID list for demo SP to use
*/	
DROP TABLE IF EXISTS [WideWorldImportersB].[dbo].[CustomerIDs];
GO

SELECT 
	DISTINCT [CustomerID], 
	DENSE_RANK() OVER (ORDER BY [CustomerID]) AS RowNum
INTO [WideWorldImportersB].[dbo].[CustomerIDs]
FROM [WideWorldImportersB].[Sales].[CustomerTransactions];
GO

DELETE FROM [WideWorldImportersB].[dbo].[CustomerIDs]
WHERE [CustomerID] IN (1, 401);
GO


/*
	Enable Query Store and clear anything that may be in there
*/
ALTER DATABASE [WideWorldImportersB] SET QUERY_STORE = ON;
GO

ALTER DATABASE [WideWorldImportersB] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE, 
	INTERVAL_LENGTH_MINUTES = 10
	);
GO

ALTER DATABASE [WideWorldImportersB] SET QUERY_STORE CLEAR;
GO


/*
	Enable Automatic Plan Correction
*/
ALTER DATABASE [WideWorldImportersB] 
	SET AUTOMATIC_TUNING (
		FORCE_LAST_GOOD_PLAN = ON
	);
GO




/*
	Create SP
*/
USE [WideWorldImportersB];
GO

DROP PROCEDURE IF EXISTS [Sales].[usp_CustomerTransactionInfo];
GO

CREATE PROCEDURE [Sales].[usp_CustomerTransactionInfo]
	@CustomerID INT
AS	

	SELECT [CustomerID], SUM([AmountExcludingTax])
	FROM [Sales].[CustomerTransactions]
	WHERE [CustomerID] = @CustomerID
	GROUP BY [CustomerID];
GO


/*
	Run external script to create workload
	Check to see what's in QS
*/


/*
	Create regression
	Check to see what's in QS
	What's in DMV?
*/
SELECT *
FROM [sys].[dm_db_tuning_recommendations];
GO

SELECT 
	[FP].[QueryId], 
	[Reason], 
	[Score],
	JSON_VALUE(state, '$.currentValue') [CurrentValue],
	[FP].[RegressedPlanId], 
	[FP].[ForcedPlanID],
	[execute_action_start_time],
	[execute_action_initiated_by],
	[revert_action_start_time],
	[revert_action_initiated_by],
	JSON_VALUE(details, '$.implementationDetails.script') [Script]
FROM [sys].[dm_db_tuning_recommendations]
CROSS APPLY OPENJSON (Details, '$.planForceDetails')
    WITH ( 
		[QueryId] INT '$.queryId',
        [RegressedPlanId] INT '$.regressedPlanId',
        [ForcedPlanID] INT '$.recommendedPlanId'
    ) AS [FP];
GO

/*
	Create recommended index
*/
CREATE NONCLUSTERED INDEX [NCI_SQLskills_CustomerTransactions]
	ON [Sales].[CustomerTransactions] ([CustomerID])
	INCLUDE ([AmountExcludingTax]);
GO

/*
	Check status
*/
SELECT 
	[FP].[QueryId], 
	[Reason], 
	[Score],
	JSON_VALUE(state, '$.currentValue') [CurrentValue],
	[FP].[RegressedPlanId], 
	[FP].[ForcedPlanID],
	[execute_action_start_time],
	[execute_action_initiated_by],
	[revert_action_start_time],
	[revert_action_initiated_by],
	JSON_VALUE(details, '$.implementationDetails.script') [Script]
FROM [sys].[dm_db_tuning_recommendations]
CROSS APPLY OPENJSON (Details, '$.planForceDetails')
    WITH ( 
		[QueryId] INT '$.queryId',
        [RegressedPlanId] INT '$.regressedPlanId',
        [ForcedPlanID] INT '$.recommendedPlanId'
    ) AS [FP];
GO


